/*
 * log.c - Logging routines
 *
 * This file is part of tcpspy, a TCP/IP connection monitor. 
 *
 * Copyright (c) 2000, 2001 Tim J. Robbins. 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $zId: log.c,v 1.5 2001/01/05 14:05:47 fyre Stab $
 * $Id: log.c,v 1.9 2002/01/04 23:07:32 wouter Exp $
 */

#include <assert.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <syslog.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

#include "rcsid.h"

/*RCSID("$zId: log.c,v 1.5 2001/01/05 14:05:47 fyre Stab $");*/
RCSID("$Id: log.c,v 1.9 2002/01/04 23:07:32 wouter Exp $");

/*
 * Exported functions.
 */
void log_set_syslog (void);
void log_set_stdout (void);
void log_set_dir (const char *dir);
void logmsg (time_t time, const char *fmt, ...);
void vlogmsg (time_t time, const char *fmt, va_list ap);
void panic (const char *fmt, ...);
void vpanic (const char *fmt, va_list ap);

/*
 * Internal helper functions.
 */
static void vlogmsg_i (int panic, time_t time, const char *fmt, va_list ap);

/*
 * Methods of logging, set via log_set_foo () family of functions. LM_SYSLOG
 * is the default.
 */
#define LM_SYSLOG	0	/* via syslog(3) */
#define LM_STDOUT	1	/* standard output */
#define LM_DIR		2	/* directory of logfiles per day */
static int logmethod = LM_SYSLOG;

/*
 * If logmethod == LM_DIR, the directory where to log.
 */
static char *logdir = NULL;

/*
 * If logmethod == LM_DIR, the day index, encoded as the number of days
 * since the 1st of January, 1970 (or time()/86400).
 */
static int today = 0;

/*
 * If logmethod == LM_DIR, the log file.
 */
static FILE *logfile = NULL;

/*
 * log_set_syslog ()
 *
 * Log via syslog(3). openlog () should be called before any of the logging
 * routines to set the ident and facility.
 */
void log_set_syslog (void)
{
	logmethod = LM_SYSLOG;
}

/*
 * log_set_stdout ()
 *
 * Log to standard output.
 */
void log_set_stdout (void)
{
	logmethod = LM_STDOUT;
}

/*
 * log_set_dir ()
 *
 * Log to a directory of log files per day.
 */
void log_set_dir (const char *dir)
{
	logmethod = LM_DIR;
	logdir = strdup (dir);
}

/*
 * logmsg ()
 * 
 * printf-style function to log messages of normal priority.
 */
void logmsg (time_t time, const char *fmt, ...)
{
	va_list ap;

	va_start (ap, fmt);
	vlogmsg (time, fmt, ap);
	va_end (ap);
}

/*
 * vlogmsg ()
 *
 * vprintf-style function to log messages of normal priority.
 */
void vlogmsg (time_t time, const char *fmt, va_list ap)
{
	vlogmsg_i (0, time, fmt, ap);	
}

/*
 * panic ()
 *
 * printf-style function to log critical errors and terminate.
 */
void panic (const char *fmt, ...)
{
	va_list ap;

	va_start (ap, fmt);
	vpanic (fmt, ap);
	/* Not reached, vpanic () exits */
	va_end (ap);
}

/*
 * vpanic ()
 *
 * vprintf-style function to log critical errors and terminate.
 */
void vpanic (const char *fmt, va_list ap)
{
	vlogmsg_i (1, 0, fmt, ap);
	exit (EXIT_FAILURE);
}

/*
 * vlogmsg_dir ()
 *
 * Write a log message to a particular file in a directory of log files, one
 * for each day.
 */
static void vlogmsg_dir (time_t now, const char *fmt, va_list ap)
{
	char timebuf[100];
	struct tm *t;

	/* switch logfiles each day */
	if (now==0)
		now = time(NULL);
	if (now/86400 != today) {
		today = now/86400;
		if (logfile != NULL)
			fclose (logfile);
		logfile = NULL;
	}

	/* figure out the new logfile */
	if (logfile == NULL) {
		char buf[64], buf2[1024];

		/* get the file name */
		t=localtime(&now);
		strftime(buf,sizeof(buf),"/tcpspy-%Y%m%d.log",t);

		/* figure out the file path */
		strncpy (buf2,logdir,sizeof(buf2));
		strncat (buf2,buf,sizeof(buf2)-strlen(buf2));

		/* open the file */
		if ((logfile = fopen(buf2,"a")) == NULL) {
			perror(buf2);
			return;
		}
	}

	/*
	 * Timestamp the message. If an error occurs, work
	 * around it so messages aren't silently lost.
	 */
	if (now == (time_t) -1)
		snprintf (timebuf, sizeof (timebuf), 
				"<unknown>");
	else if ((t = localtime (&now)) == NULL)
		snprintf (timebuf, sizeof (timebuf), "EPOCH+%u",
			      	(unsigned int) now);
	else if (strftime (timebuf, sizeof (timebuf), 
				"%b %d %H:%M:%S", t) == 0)
		snprintf (timebuf, sizeof (timebuf), "%u", 
				(unsigned int) now);
	fprintf (logfile, "%s: ", timebuf);

	vfprintf (logfile, fmt, ap);
	fputc ('\n', logfile);
	fflush (logfile);
}

/*
 * vlogmsg_i ()
 *
 * Internal routine called by logmsg (), vlogmsg (), panic () and vpanic ().
 * Logs message by specified method.
 */
static void vlogmsg_i (int panic, time_t now, const char *fmt, va_list ap)
{
	switch (logmethod) {
		case LM_SYSLOG:
			vsyslog (panic == 0 ? LOG_NOTICE : LOG_ERR, fmt, ap);
			break;

		case LM_STDOUT:	
			{
			char timebuf[100];
			struct tm *t;
			FILE *fp;

			/*
			 * Timestamp the message. If an error occurs, work
			 * around it so messages aren't silently lost.
			 */
			if ((now==0) && ((now = time (NULL)) == (time_t) -1))
				snprintf (timebuf, sizeof (timebuf), 
						"<unknown>");
			else if ((t = localtime (&now)) == NULL)
				snprintf (timebuf, sizeof (timebuf), "EPOCH+%u",
					      	(unsigned int) now);
			else if (strftime (timebuf, sizeof (timebuf), 
						"%b %d %H:%M:%S", t) == 0)
				snprintf (timebuf, sizeof (timebuf), "%u", 
						(unsigned int) now);
			/*
			 * Normal messages go to stdout, panic messages go
			 * to stderr.
			 */
			fp = (panic == 0) ? stdout : stderr;
			
			fprintf (fp, "%s: ", timebuf);

			vfprintf (fp, fmt, ap);
			fputc ('\n', fp);
			fflush (fp);
			}
			break;
		case LM_DIR:
			vlogmsg_dir(now,fmt,ap);
			break;
		default:
			assert (0);
	}
}
