/*
 * tcpmon.h - Linux 2.2 kernel TCP monitor (part of tcpspy)
 * Copyright (c) 2000, 2001 Tim J. Robbins <tim@robbins.dropbear.id.au>
 *
 * Mostly rewritten by Wouter Coene <wouter@inebriated.demon.nl>
 * Copyright (C) Wouter Coene <wouter@inebriated.demon.nl>, 2001, 2002
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Brief overview:
 * 	Creates a file /proc/tcpmon from which messages are read by a
 * 	userspace process. These messages are written when a connect(),
 * 	listen() or accept() call occurs, and contain information about
 * 	the current process, and the local and remote network addresses
 * 	(if applicable).
 *
 * To do:
 *	- Ensure that this module is SMP-safe, particularly the replacement
 *	socketcall.
 *
 * Contains code from various places in the Linux kernel source:
 * 	linux/kernel/printk.c	Linus Torvalds
 * 	linux/net/socket.c	Orest Zborowski, Ross Biro, Fred N. van Kempen
 * 	linux/fs/proc/kmsg.c	Linus Torvalds
 *
 * $Id: tcpmon.h,v 1.7 2002/01/04 22:55:19 wouter Exp $
 */

#ifndef __TCPMON_H__
#define __TCPMON_H__

/* generic definitions */
#ifndef TRUE
# define TRUE	(1==1)
#endif /* TRUE */

#ifndef FALSE
# define FALSE	(1==0)
#endif /* FALSE */

/* stringification */
#define STR(x)		STR2(x)
#define STR2(x)	#x

/* tcpmon device minor device number */
#define TCPMON_DEV_MINOR	240

/* path to tcpmon device */
#define TCPMON_DEV_NAME		"tcpmon"
#define TCPMON_DEV_PATH		"/dev/" TCPMON_DEV_NAME

/* ioctls */
#define TCPMON_IOCTL_FETCH	10	/* fetch a log message */
#define TCPMON_IOCTL_ENABLE	11	/* enable connection logging */
#define TCPMON_IOCTL_DISABLE	12	/* disable connection logging */
#define TCPMON_IOCTL_STATUS	13	/* fetch module status */

/* command name size */
#define TCPMON_COMM_SIZE	32

/* connection states */
#define TCPMON_STATE_CLOSED	0
#define TCPMON_STATE_CONNECTING	1
#define TCPMON_STATE_CONNECTED	2
#define TCPMON_STATE_CONNFAIL	3
#define TCPMON_STATE_CLOSING	4

/* log message structure */
typedef struct tcpmon_log_message {
	time_t		time;			/* the time stamp */
	int		status,			/* connection state */
			pid,			/* process id */
			uid,			/* user id */
			error;			/* error code */
	char		comm[TCPMON_COMM_SIZE];	/* command name */
	struct in_addr	local,			/* local address */
			remote;			/* remote address */
	int		lport,			/* local port */
			rport;			/* remote port */

#ifdef __KERNEL__
	struct tcpmon_log_message *prev, *next;	/* message queue linkage */
#else /* ! __KERNEL__ */
	void		*dummy1, *dummy2;
#endif /* __KERNEL__ */
} tcpmon_log_message_t;

/* log state strings */
char *tcpmon_state[]={
	"CLOSED",
	"CONNECTING",
	"CONNECTED",
	"CONNFAIL",
	"CLOSING",
	"UNKNOWN"
};

/* kernel module status */
typedef struct tcpmon_status {
	char		version[32];	/* kernel module version */
	int		enabled,	/* logging enabled or not? */
			state_size,	/* state table size */
			state_load,	/* entries in state table */
			queue_length;	/* queued messages */
	int		connections,	/* number of connections handled */
			messages;	/* number of log messages */
	int		memory;		/* memory usage in bytes */
} tcpmon_status_t;

#endif /* __TCPMON_H__ */

/* end tcpmon.h */
